Index: html/docs/objectdefinitions.html
===================================================================
RCS file: /cvsroot/nagios/nagios/html/docs/objectdefinitions.html,v
retrieving revision 1.19
diff -u -p -r1.19 objectdefinitions.html
--- html/docs/objectdefinitions.html	4 Nov 2008 22:28:40 -0000	1.19
+++ html/docs/objectdefinitions.html	12 Dec 2008 22:07:19 -0000
@@ -157,6 +157,7 @@ Note:  Directives in red are required, w
 <tr><td></td><td class="Optional">statusmap_image</td><td class="Optional"><i>image_file</i></td></tr>
 <tr><td></td><td class="Optional">2d_coords</td><td class="Optional"><i>x_coord,y_coord</i></td></tr>
 <tr><td></td><td class="Optional">3d_coords</td><td class="Optional"><i>x_coord,y_coord,z_coord</i></td></tr>
+<tr><td></td><td class="Optional">services</td><td class="Optional"><i>servicelist</i></td></tr>
 
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td colspan=2 class="Definition">}</td></tr>
 </table>
@@ -452,6 +453,58 @@ This variable is used to define coordina
 This variable is used to define coordinates to use when drawing the host in the <a href="cgis.html#statuswrl_cgi">statuswrl</a> CGI.  Coordinates can be positive or negative real numbers.  The origin for drawing is (0.0,0.0,0.0).  For reference, the size of the host cubes drawn is 0.5 units on each side (text takes a little more space).  The coordinates you specify here are used as the center of the host cube.
 </td>
 </tr>
+<tr>
+<td valign="top"><strong>services</strong>:</td>
+<td>
+This directive can be used to automatically instantiate <a href="#service">service</a>
+definitions for this host based on the indicated service templates.
+Multiple service template names should be separated by commas.
+<br>
+This directive may be used as an alternative to (or in addition to) using <a href="#service">service</a> definitions for this host.
+<br>
+The following sequence:
+<br>
+<pre>
+    define host{
+        host_name                      bogus
+        address                        192.168.1.254
+        services                       template-check-disk-sda, template-check-disk-sdb
+        ...
+    }
+</pre>
+
+is exactly equivalent to the following sequence:
+
+<pre>
+    define host{
+        host_name                      bogus
+        address                        192.168.1.254
+        ...
+    }
+    define service {
+        host_name                      bogus
+        use                            template-check-disk-sda
+    }
+    define service {
+        host_name                      bogus
+        use                            template-check-disk-sdb
+    }
+</pre>
+If the first character of the <b>services</b> list is a <code><b>+</b></code> (plus),
+the list of services is added to any previously defined <b>services</b>.
+Accordingly, the following sequence is identical to the example above:
+<br>
+<pre>
+    define host{
+        host_name                      bogus
+        address                        192.168.1.254
+        services                       +template-check-disk-sda
+        services                       +template-check-disk-sdb
+        ...
+    }
+</pre>
+</td>
+</tr>
 </table>
 
 
Index: xdata/xodtemplate.c
===================================================================
RCS file: /cvsroot/nagios/nagios/xdata/xodtemplate.c,v
retrieving revision 1.182
diff -u -p -r1.182 xodtemplate.c
--- xdata/xodtemplate.c	30 Nov 2008 17:22:59 -0000	1.182
+++ xdata/xodtemplate.c	12 Dec 2008 22:07:20 -0000
@@ -1549,6 +1549,7 @@ int xodtemplate_begin_object_definition(
 		new_host->retain_nonstatus_information=TRUE;
 		new_host->have_retain_nonstatus_information=FALSE;
 		new_host->custom_variables=NULL;
+		new_host->host_auto_services=NULL;
 		new_host->has_been_resolved=FALSE;
 		new_host->register_object=TRUE;
 
@@ -1990,6 +1991,50 @@ int xodtemplate_begin_object_definition(
 	return result;
         }
 
+/*
+ * Utility function:
+ *
+ * Handle string elements that allow "+" to add new values.
+ *
+ * Arguments:
+ *	property	A pointer to where the property value should be stored
+ *	value		The value for the property
+ *	allowAdd	Boolean. If true, a leading "+" will concatenate this value to
+ *			any previous property value. The old and new values will be
+ *			separated with a "," (comma). A leading "+" is always removed.
+ *			If false, a new value will replace any previous property value.
+ */
+static int _xodtemplate_set_string_property( char ** property,  char * value, int allowAdd )
+{
+	if(strcmp(value,XODTEMPLATE_NULL)){
+
+		if ( allowAdd && ( value[0] == '+' ) ) {
+			/* Add new value to old string */
+			if(*property == NULL) {
+				*property = (char *)strdup( value+1 );
+				}
+			else {
+				value[0] = ',';
+
+				*property = (char *)realloc( *property,
+							     ( strlen( *property ) + strlen( value ) + 2 ) );
+				if(*property == NULL)
+					return ERROR;
+
+				strcat( *property, value );
+				}
+			}
+		else {
+			/* prevent unintentional leaks */
+			my_free(*property);
+			*property = (char *)strdup(value);
+			if(*property == NULL)
+				return ERROR;
+			}
+		}
+
+	return OK;
+}
 
 
 /* adds a property to an object definition */
@@ -3665,6 +3710,9 @@ int xodtemplate_add_object_property(char
 			temp_host->retain_nonstatus_information=(atoi(value)>0)?TRUE:FALSE;
 			temp_host->have_retain_nonstatus_information=TRUE;
 		        }
+		else if(!strcmp(variable,"services")){
+			result = _xodtemplate_set_string_property( &temp_host->host_auto_services, value, TRUE );
+		        }
 		else if(!strcmp(variable,"register"))
 			temp_host->register_object=(atoi(value)>0)?TRUE:FALSE;
 		else if(variable[0]=='_'){
@@ -4702,15 +4750,129 @@ int xodtemplate_add_object_property(char
         }
 
 
+/* Handles the "services" statement on a host definition.
+ * For each service name in the "services" list, auto-generate services
+ * as if we had also read:
+ *
+ *	define service {
+ *	    host_name HOSTNAME
+ *	    use TEMPLATE
+ *	}
+ */
+static int _xodtemplate_create_host_auto_services(xodtemplate_host *this_host){
+	char *servicelist;
+	char *servicelisttofree;
+	char *service=NULL;
+	int result=OK;
+
+	/* Ignore hosts without a "services" statement or templates.
+	 * Only real host entries should auto-generate the service entries.
+	 *
+	 * FIXME: the auto-creation should happen after resolving the host
+	 *	  so "services" could be inherited from a template.
+	 */
+	if(this_host->host_auto_services==NULL || !this_host->register_object){
+		/* Nothing to do */
+		return result;
+		}
+
+	/* Duplicate the list to avoid destroying it in the process of parsing it.
+	 * Save the pointer so we can free this area later (my_strsep() sets the input pointer to NULL,
+	 * leading to memory leaks)
+	 */
+	servicelist = strdup(this_host->host_auto_services);
+	servicelisttofree = servicelist;
+
+	if(servicelist == NULL){
+		result=ERROR;
+	}
+
+	while((result == OK) && (NULL != (service=my_strsep(&servicelist,",")))){
+		char *line;
+
+		strip(service);
+
+		if(service[0] == '\x0'){
+			continue;
+			}
+
+		/* "define service {" */
+		result = xodtemplate_begin_object_definition("service", READ_ALL_OBJECT_DATA,
+							     this_host->_config_file, this_host->_start_line);
+		if(result != OK){
+			break;
+			}
+
+		/* "host_name HOSTNAME" */
+		asprintf(&line, "host_name %s", this_host->host_name);
+		result = xodtemplate_add_object_property(line, READ_ALL_OBJECT_DATA);
+		my_free(line);
+		if(result != OK){
+			break;
+			}
+
+		/* "use SERVICE_TEMPLATE" */
+		asprintf(&line, "use %s", service);
+		result = xodtemplate_add_object_property(line, READ_ALL_OBJECT_DATA);
+		my_free(line);
+		if(result != OK){
+			break;
+			}
+
+		/* "}" */
+		result = xodtemplate_end_object_definition(READ_ALL_OBJECT_DATA);
+		if(result != OK){
+			break;
+			}
+		}
+
+#ifdef NSCORE
+	if(result != OK){
+		if(service==NULL){
+			logit(NSLOG_CONFIG_ERROR,TRUE,
+			      "Error: Could not add auto service definitions for host '%s', file '%s', line %d.\n",
+			      this_host->host_name,
+			      xodtemplate_config_file_name(this_host->_config_file),
+			      this_host->_start_line);
+			}
+		else{
+			logit(NSLOG_CONFIG_ERROR,TRUE,
+			      "Error: Could not add auto service definition '%s' for host '%s', file '%s', line %d.\n",
+			      service,
+			      this_host->host_name,
+			      xodtemplate_config_file_name(this_host->_config_file),
+			      this_host->_start_line);
+			}
+		}
+#endif
+
+	my_free(servicelisttofree);
+
+	return result;
+	}
+
+
 
 /* completes an object definition */
 int xodtemplate_end_object_definition(int options){
+	xodtemplate_host *this_host=NULL;
 	int result=OK;
 
+	/* If this was a host definition, save the object. */
+	if(xodtemplate_current_object_type == XODTEMPLATE_HOST){
+		this_host = (xodtemplate_host *) xodtemplate_current_object;
+	}
 
+	/* Finish the object */
 	xodtemplate_current_object=NULL;
 	xodtemplate_current_object_type=XODTEMPLATE_NONE;
 
+
+	/* If it was a host definition, auto-generate any requested services. */
+	if(this_host != NULL){
+		result = _xodtemplate_create_host_auto_services(this_host);
+	}
+
 	return result;
         }
 
@@ -7958,6 +8120,8 @@ int xodtemplate_resolve_host(xodtemplate
 			this_host->retain_nonstatus_information=template_host->retain_nonstatus_information;
 			this_host->have_retain_nonstatus_information=TRUE;
 	                }
+		if(this_host->host_auto_services==NULL && template_host->host_auto_services!=NULL)
+			this_host->host_auto_services=(char *)strdup(template_host->host_auto_services);
 
 		/* apply missing custom variables from template host... */
 		for(temp_customvariablesmember=template_host->custom_variables;temp_customvariablesmember!=NULL;temp_customvariablesmember=temp_customvariablesmember->next){
Index: xdata/xodtemplate.h
===================================================================
RCS file: /cvsroot/nagios/nagios/xdata/xodtemplate.h,v
retrieving revision 1.70
diff -u -p -r1.70 xodtemplate.h
--- xdata/xodtemplate.h	30 Nov 2008 17:22:59 -0000	1.70
+++ xdata/xodtemplate.h	12 Dec 2008 22:07:20 -0000
@@ -283,6 +283,12 @@ typedef struct xodtemplate_host_struct{
 	int       retain_nonstatus_information;
 	xodtemplate_customvariablesmember *custom_variables;
 
+	/*
+	 * Comma-separated list of service templates to be automatically
+	 * instantiated for this host.
+	 */
+	char      *host_auto_services;
+
 	int       have_display_name;
 	int       have_parents;
 	int       have_host_groups;
